
/* Chrysalide - Outil d'analyse de fichiers binaires
 * ##FILE## - traduction d'instructions ARMv7
 *
 * Copyright (C) 2015 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


@title ASR (immediate)

@desc Arithmetic Shift Right (immediate) shifts a register value right by an immediate number of bits, shifting in copies of its sign bit, and writes the result to the destination register. It can optionally update the condition flags based on the result.

@encoding (t1) {

	@half 0 0 0 1 0 imm5(5) Rm(3) Rd(3)

	@syntax "asrs" <reg_D> <reg_M> <shift_imm>

	@conv {

		reg_D = Register(Rd)
		reg_M = Register(Rm)
		shift_imm = DecodeImmShift('10', imm5)

	}

}

@encoding (T2) {

	@word 1 1 1 0 1 0 1 0 0 1 0 S(1) 1 1 1 1 0 imm3(3) Rd(4) imm2(2) 1 0 Rm(4)

	@syntax <reg_D> <reg_M> <shift_imm>

	@conv {

		reg_D = Register(Rd)
		reg_M = Register(Rm)
		setflags = (S == '1')
		shift_imm = DecodeImmShift('10', imm3:imm2)

	}

	@rules {

		if (setflags); chk_call ExtendKeyword("s")
		chk_call ExtendKeyword(".w")

	}

}

@encoding (A1) {

	@word cond(4) 0 0 0 1 1 0 1 S(1) 0 0 0 0 Rd(4) imm5(5) 1 0 0 Rm(4)

	@syntax <reg_D> <reg_M> <shift_imm>

	@conv {

		reg_D = Register(Rd)
		reg_M = Register(Rm)
		setflags = (S == '1')
		shift_imm = DecodeImmShift('10', imm5)

	}

	@rules {

		if (setflags); chk_call ExtendKeyword("s")
		chk_call StoreCondition(cond)

	}

}

